/* read-blockmap.c
 * Companion to map-compiler.lisp, reads in a blockmap file generated
 * by that program.
 *
 * Copyright 2011 Jonathan L. Fischer <jonathan@mohiji.org>.  All rights reserved.
 */

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>

const uint8_t BLOCKMAP_MAGIC = 123;
const uint8_t BLOCKMAP_VERSION = 1;

struct BlockMapFile
{
    uint8_t  magic;              // Identify the file type
    uint8_t  version;            // Cause you know you'll need it
    char     title[256];         // Fixed length strings make life happy
    char     background[256];
    uint32_t light_color;        // ARGB light color ready to drop into OpenGL
    float    light_direction[3]; // 32-bit floats written out as uint32s
    uint8_t  num_layers;         // Dimensions of the layers
    uint8_t  width;
    uint8_t  height;
    uint8_t  layers[1];          // num_layers * width * height entries
}__attribute__((packed));        // Bloody field alignment

int main(int argc, char **argv)
{
    struct stat file_stat;
    FILE *f;
    char *data;
    char *file_name;
    struct BlockMapFile *blockmap;
    int i, x, y, offset;
    
    if (argc < 2) 
    {
        printf("Usage: read-blockmap <mapfile>\n");
        return -1;
    }
    
    file_name = argv[1];
    if (stat(file_name, &file_stat) != 0) 
    {
        printf("Unable to stat file %s\n", file_name);
        return -1;
    }
    
    f = fopen(file_name, "rb");
    if (f == NULL) 
    {
        printf("Unable to open file %s\n", file_name);
        return -1;
    }
    
    data = calloc(1, file_stat.st_size);
    fread(data, 1, file_stat.st_size, f);
    fclose(f);
    
    blockmap = (struct Blockmap*)data;
    printf("Here's your map:\n");
    printf("  Title: %s\n", blockmap->title);
    printf("  Background: %s\n", blockmap->background);
    printf("  Light Color: (%x)\n", blockmap->light_color);
    printf("  Light Direction: (%f, %f, %f)\n", 
           blockmap->light_direction[0],
           blockmap->light_direction[1],
           blockmap->light_direction[2]);
    printf("  Number of Layers: %d\n", blockmap->num_layers);
    printf("  Dimensions: %d, %d\n", blockmap->width, blockmap->height);
    printf("  Layers:\n");
    
    offset = 0;
    for (i = 0;i < blockmap->num_layers; i++) 
    {
        for (y = 0;y < blockmap->height; y++) 
        {
            printf("    ");
            for (x = 0;x < blockmap->width; x++) 
            {
                printf("%d ", blockmap->layers[offset]);
                offset++;
            }
            printf("\n");
        }
        printf("\n");
    }
    
    // Done
    free(data);
    return 0;
}
